<?php

// PHP code for displaying an unzipped PNG file
// or a PNG image generated by "dot" from an unzipped TXT file.
//
// Copyright (C) 2006-2011 by Pawel Pilarczyk, http://www.pawelpilarczyk.com/
//
// This file is part of my research software package.  This is free software;
// you can redistribute it and/or modify it under the terms of the GNU Affero
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public
// License along with this software; see the file "license.txt".
// If not, see <http://www.gnu.org/licenses/>.

// Started in 2006. Last revision: June 13, 2011.


require ("config.php");


// acquire the requested parameters
$zipfile = escapeshellcmd (stripslashes
	(array_key_exists ('zip', $_REQUEST) ? $_REQUEST['zip'] : ''));
$txtfile = escapeshellcmd (stripslashes
	(array_key_exists ('txt', $_REQUEST) ? $_REQUEST['txt'] : ''));
$pngfile = escapeshellcmd (stripslashes
	(array_key_exists ('png', $_REQUEST) ? $_REQUEST['png'] : ''));
$maxsize = escapeshellcmd (stripslashes
	(array_key_exists ('max', $_REQUEST) ? $_REQUEST['max'] : ''));
$mark = escapeshellcmd (stripslashes
	(array_key_exists ('mark', $_REQUEST) ? $_REQUEST['mark'] : ''));

// measure the number of successfully passed stages
$success_count = 0;
$success_ok = 3;

// send the header indicating the PNG image file type
header ("Content-type: image/png");

// if no size limit requested and the PNG file is just to be unzipped
if ((empty ($maxsize) || ($maxsize == 0)) && !empty ($pngfile) &&
	($mark === "0"))
{
	while (1)
	{
		// if no zip file name was given then this is wrong
		if (empty ($zipfile))
			break;
		++ $success_count;

		// if the zip file does not exist then this is bad
		if (!file_exists ($zipfile))
			break;
		++ $success_count;

		// unzip the picture from the file and send it to the output
		passthru ("unzip -p ${zipfile} ${pngfile}", $returnvalue);
		if ($returnvalue)
			break;
		++ $success_count;

		break;
	}
}

// otherwise proceed with a more sophisticated procedure
else
{
	while (1)
	{
		// if insufficiently many file names are given then stop it
		if (empty ($zipfile))
			break;
		if (empty ($pngfile) && empty ($txtfile))
			break;
		++ $success_count;

		// if the zip file does not exist then quit
		if (!file_exists ($zipfile))
			break;

		// prepare a temporary subdirectory to use
		$tmpdir = tempnam ("", "PNGUNZIP");
		@unlink ($tmpdir);
		@rmdir ($tmpdir);
		if (@mkdir ($tmpdir, 0700) === false)
			break;

		// uncompress the requested file
		$thefile = empty ($pngfile) ? $txtfile : $pngfile;
		$output = exec ("unzip ${zipfile} ${thefile} -d ${tmpdir}");
		if (!empty ($output))
			++ $success_count;

		// convert the given TXT file into a PNG one
		if (empty ($pngfile))
		{
			$pngfile = $txtfile . ".png";
			$output = exec ("dot ${tmpdir}/${txtfile} -T png " .
				"-o ${tmpdir}/${pngfile} ");
			unlink ("${tmpdir}/${txtfile}");
		}

		// mark the PNG file unless requested to leave it untouched
		if ($mark !== "0")
		{
			if (($mark === "v") || ($mark === "h"))
				$markopt = "-${mark} ";
			elseif (empty ($txtfile))
				$markopt = "-h ";
			else
				$markopt = "-v ";
			exec ("${bindir}pngmark " .
				"${markopt}${tmpdir}/${pngfile}");
		}

		// if the PNG file does not exist then this is bad
		if (!file_exists ("${tmpdir}/${pngfile}"))
		{
			rmdir ($tmpdir);
			break;
		}

		// set the name of the actual image file to display
		$imgfile = $pngfile;

		// determine the size of the PNG file
		list ($width, $height, $type, $attr) =
			getimagesize ("${tmpdir}/${pngfile}");

		// create a smaller file if the size is too large
		if (!empty ($maxsize) &&
			(($width > $maxsize) || ($height > $maxsize)))
		{
			$smallfile = $pngfile . "s";
			exec ("${bindir}pngsmall " .
				"${tmpdir}/${pngfile} " .
				"${tmpdir}/${smallfile} " .
				"-w ${maxsize} -h ${maxsize}");
			if (file_exists ("${tmpdir}/${smallfile}"))
			{
				unlink ("${tmpdir}/${pngfile}");
				$imgfile = $smallfile;
			}
		}

		// open the image file to display
		@$file = fopen ("${tmpdir}/${imgfile}", "rb");
		if ($file === false)
		{
			rmdir ("${tmpdir}");
			break;
		}
		++ $success_count;

		// read the entire image file into the memory
		$imgdata = fread ($file, 2 * 1024 * 1024);
		fclose ($file);

		// remove the image file and the temporary directory
		unlink ("${tmpdir}/${imgfile}");
		rmdir ("${tmpdir}");

		// copy the image data to the output stream
		echo $imgdata;
		break;
	}
}

// show an error image if some kind of failure was encountered
if ($success_count < $success_ok)
{
	// determine the image, based on the success rate [unused]
	switch ($success_count)
	{
		case 2: $errorfile = "pngerror.png"; break;
		case 1: $errorfile = "pngerror.png"; break;
		default: $errorfile = "pngerror.png"; break;
	}

	// read the error image file and copy it to the output
	$file = fopen ($errorfile, "rb");
	$imgdata = fread ($file, 2 * 1024 * 1024);
	fclose ($file);
	echo $imgdata;
}

//echo "<html><body>txtname=${txtname}<br/>pngname=${pngname}<br/>${graph}</body></html>";

?>