/////////////////////////////////////////////////////////////////////////////
///
/// @file typerect.h
///
/// Data type for the rectangular (Lohner-type) set from CAPD.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2014 by Pawel Pilarczyk.
//
// This file is part of my research software package. This is free software:
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; see the file "license.txt". If not,
// please, see <https://www.gnu.org/licenses/>.

// Started on July 18, 2012. Last revision: October 25, 2013.


#ifndef _CMGRAPHS_TYPERECT_H_
#define _CMGRAPHS_TYPERECT_H_


// include all the necessary header files from the CAPD library
#ifdef _CAPD29_
#include "capd/vectalg/lib.h"
#include "capd/dynset/lib.h"
#else
//#include "capd/capdlib.h"
#define CAPD_USER_NAMESPACE capd
#define CAPD_DEFAULT_DIMENSION spaceDim
#include "capd/fdcapdlib.h"
#undef CAPD_USER_NAMESPACE
#undef CAPD_DEFAULT_DIMENSION
#endif


// --------------------------------------------------
// ------------------- data types -------------------
// --------------------------------------------------

/// The type of the rectangular set to use.
#ifdef _CAPD29_
typedef capd::C0Rect2Set RectSetType;
#else
//typedef capd::C0Rect2Set RectSetType;
typedef capd::C0HORect2Set RectSetType;
#define PREDICTOR_CORRECTOR
#endif

/// The interval vector type for creating the rectangular set
/// by means of its center + box.
typedef capd::IVector IntervalVectorType;


#endif // _CMGRAPHS_TYPERECT_H_

